(function(){
if( navigator.userAgent.search(/Firefox/gi) == -1 ) {

    const myDEBUG = false; /* set to false to suppress myDebugFunc messages */
    const myPANELS_RDF_FILE  = "UPnls"; /* directory services property to find panels.rdf */

    const myCONTAINER_CONTRACTID = "@mozilla.org/rdf/container;1";
    const myDIR_SERV_CONTRACTID  = "@mozilla.org/file/directory_service;1"
    const myIO_SERV_CONTRACTID   = "@mozilla.org/network/io-service;1";
    const mynsISupports      = Components.interfaces.nsISupports;
    const mynsIFactory       = Components.interfaces.nsIFactory;
    const mynsISidebar       = Components.interfaces.nsISidebar;
    const mynsIRDFContainer  = Components.interfaces.nsIRDFContainer;
    const mynsIProperties    = Components.interfaces.nsIProperties;
    const mynsIRDFRemoteDataSource = Components.interfaces.nsIRDFRemoteDataSource;
    const mynsIClassInfo = Components.interfaces.nsIClassInfo;

    // copied from nsSidebar.js and removed the security check!

    /* static functions */
    if (myDEBUG)
        myDebugFunc = function (s) { dump("-*- urlparams sidebar component: " + s + "\n"); }
    else
        myDebugFunc = function (s) {}

    function mySidebar()
    {
        const RDF_CONTRACTID = "@mozilla.org/rdf/rdf-service;1";
        const nsIRDFService = Components.interfaces.nsIRDFService;

        this.rdf = Components.classes[RDF_CONTRACTID].getService(nsIRDFService);
        this.datasource_uri = myGetSidebarDatasourceURI(myPANELS_RDF_FILE);
        myDebugFunc('datasource_uri is ' + this.datasource_uri);
        this.resource = 'urn:sidebar:current-panel-list';
        this.datasource = this.rdf.GetDataSource(this.datasource_uri);

        const PROMPTSERVICE_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
        const nsIPromptService = Components.interfaces.nsIPromptService;
        this.promptService =
            Components.classes[PROMPTSERVICE_CONTRACTID].getService(nsIPromptService);
    }

    mySidebar.prototype.nc = "http://home.netscape.com/NC-rdf#";

    mySidebar.prototype.isPanel =
    function (aContentURL)
    {
        var container =
            Components.classes[myCONTAINER_CONTRACTID].createInstance(mynsIRDFContainer);

        container.Init(this.datasource, this.rdf.GetResource(this.resource));

        /* Create a resource for the new panel and add it to the list */
        var panel_resource =
            this.rdf.GetResource("urn:sidebar:3rdparty-panel:" + aContentURL);

        return (container.IndexOf(panel_resource) != -1);
    }

    /* decorate prototype to provide ``class'' methods and property accessors */
    mySidebar.prototype.addPanel =
    function (aTitle, aContentURL, aCustomizeURL)
    {
        myDebugFunc("addPanel(" + aTitle + ", " + aContentURL + ", " +
              aCustomizeURL + ")");

        return this.addPanelInternal(aTitle, aContentURL, aCustomizeURL, false);
    }

    mySidebar.prototype.addPersistentPanel =
    function(aTitle, aContentURL, aCustomizeURL)
    {
        myDebugFunc("addPersistentPanel(" + aTitle + ", " + aContentURL + ", " +
               aCustomizeURL + ")\n");

        return this.addPanelInternal(aTitle, aContentURL, aCustomizeURL, true);
    }

    mySidebar.prototype.addPanelInternal =
    function (aTitle, aContentURL, aCustomizeURL, aPersist)
    {
        // Create a "container" wrapper around the current panels to
        // manipulate the RDF:Seq more easily.
        var panel_list = this.datasource.GetTarget(this.rdf.GetResource(this.resource), this.rdf.GetResource(mySidebar.prototype.nc+"panel-list"), true);
        if (panel_list) {
            panel_list.QueryInterface(Components.interfaces.nsIRDFResource);
        } else {
            // Datasource is busted. Start over.
            myDebugFunc("Sidebar datasource is busted\n");
        }

        var container = Components.classes[myCONTAINER_CONTRACTID].createInstance(mynsIRDFContainer);
        container.Init(this.datasource, panel_list);

        /* Create a resource for the new panel and add it to the list */
        var panel_resource =
            this.rdf.GetResource("urn:sidebar:3rdparty-panel:" + aContentURL);
        var panel_index = container.IndexOf(panel_resource);
        var stringBundle, brandStringBundle, titleMessage, dialogMessage;
        if (panel_index != -1)
        {
            try {
                stringBundle = srGetStrBundle("chrome://communicator/locale/sidebar/sidebar.properties");
                brandStringBundle = srGetStrBundle("chrome://branding/locale/brand.properties");
                if (stringBundle) {
                    sidebarName = brandStringBundle.GetStringFromName("sidebarName");
                    titleMessage = stringBundle.GetStringFromName("dupePanelAlertTitle");
                    dialogMessage = stringBundle.GetStringFromName("dupePanelAlertMessage");
                    dialogMessage = dialogMessage.replace(/%url%/, aContentURL);
                    dialogMessage = dialogMessage.replace(/%name%/, sidebarName);
                }
            }
            catch (e) {
                titleMessage = "Sidebar";
                dialogMessage = aContentURL + " already exists in Sidebar.";
            }

            this.promptService.alert(null, titleMessage, dialogMessage);

            return;
        }

        try {
            stringBundle = srGetStrBundle("chrome://communicator/locale/sidebar/sidebar.properties");
            brandStringBundle = srGetStrBundle("chrome://branding/locale/brand.properties");
            if (stringBundle) {
                sidebarName = brandStringBundle.GetStringFromName("sidebarName");
                titleMessage = stringBundle.GetStringFromName("addPanelConfirmTitle");
                dialogMessage = stringBundle.GetStringFromName("addPanelConfirmMessage");
                if (aPersist)
                {
                    var warning = stringBundle.GetStringFromName("persistentPanelWarning");
                    dialogMessage += "\n" + warning;
                }
                dialogMessage = dialogMessage.replace(/%title%/, aTitle);
                dialogMessage = dialogMessage.replace(/%url%/, aContentURL);
                dialogMessage = dialogMessage.replace(/#/g, "\n");
                dialogMessage = dialogMessage.replace(/%name%/g, sidebarName);
            }
        }
        catch (e) {
            titleMessage = "Add Tab to Sidebar";
            dialogMessage = "Add the Tab '" + aTitle + "' to Sidebar?\n\n" + "Source: " + aContentURL;
        }

        var rv = this.promptService.confirm(null, titleMessage, dialogMessage);

        if (!rv)
            return;

        /* Now make some sidebar-ish assertions about it... */
        this.datasource.Assert(panel_resource,
                               this.rdf.GetResource(this.nc + "title"),
                               this.rdf.GetLiteral(aTitle),
                               true);
        this.datasource.Assert(panel_resource,
                               this.rdf.GetResource(this.nc + "content"),
                               this.rdf.GetLiteral(aContentURL),
                               true);
        if (aCustomizeURL)
            this.datasource.Assert(panel_resource,
                                   this.rdf.GetResource(this.nc + "customize"),
                                   this.rdf.GetLiteral(aCustomizeURL),
                                   true);
        var persistValue = aPersist ? "true" : "false";
        this.datasource.Assert(panel_resource,
                               this.rdf.GetResource(this.nc + "persist"),
                               this.rdf.GetLiteral(persistValue),
                               true);

        container.AppendElement(panel_resource);

        // Use an assertion to pass a "refresh" event to all the sidebars.
        // They use observers to watch for this assertion (in sidebarOverlay.js).
        this.datasource.Assert(this.rdf.GetResource(this.resource),
                               this.rdf.GetResource(this.nc + "refresh"),
                               this.rdf.GetLiteral("true"),
                               true);
        this.datasource.Unassert(this.rdf.GetResource(this.resource),
                                 this.rdf.GetResource(this.nc + "refresh"),
                                 this.rdf.GetLiteral("true"));

        /* Write the modified panels out. */
        this.datasource.QueryInterface(mynsIRDFRemoteDataSource).Flush();

    }

    // property of mynsIClassInfo
    mySidebar.prototype.flags = mynsIClassInfo.DOM_OBJECT;

    // property of mynsIClassInfo
    mySidebar.prototype.classDescription = "Sidebar";

    // method of mynsIClassInfo
    mySidebar.prototype.getInterfaces = function(count) {
        var interfaceList = [mynsISidebar, mynsIClassInfo];
        count.value = interfaceList.length;
        return interfaceList;
    }

    // method of mynsIClassInfo
    mySidebar.prototype.getHelperForLanguage = function(count) {return null;}

    mySidebar.prototype.QueryInterface =
    function (iid) {
        if (iid.equals(mynsISidebar) ||
            iid.equals(mynsIClassInfo) ||
            iid.equals(mynsISupports))
            return this;

        Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
        return null;
    }

    function myGetSidebarDatasourceURI(panels_file_id)
    {
        try
        {
            /* use the fileLocator to look in the profile directory
             * to find 'panels.rdf', which is the
             * database of the user's currently selected panels. */
            var directory_service = Components.classes[myDIR_SERV_CONTRACTID].getService(Components.interfaces.nsIProperties);

            /* if <profile>/panels.rdf doesn't exist, get will copy
             *bin/defaults/profile/panels.rdf to <profile>/panels.rdf */
            var sidebar_file = directory_service.get(panels_file_id, Components.interfaces.nsIFile);

            if (!sidebar_file.exists())
            {
                /* this should not happen, as GetFileLocation() should copy
                 * defaults/panels.rdf to the users profile directory */
                myDebugFunc("sidebar file does not exist");
                return null;
            }

            var io_service = Components.classes[myIO_SERV_CONTRACTID].getService(Components.interfaces.nsIIOService);
            var file_handler = io_service.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
            var sidebar_uri = file_handler.getURLSpecFromFile(sidebar_file);
            myDebugFunc("sidebar uri is " + sidebar_uri);
            return sidebar_uri;
        }
        catch (ex)
        {
            /* this should not happen */
            myDebugFunc("caught " + ex + " getting sidebar datasource uri");
            return null;
        }
    }


    // add sidebar
    var oSidebar = new mySidebar();
    if( oSidebar ) {
        oSidebar.addPanel("UrlParams", "chrome://urlparams/content/sidebar.xul", "chrome://urlparams/content/settings.xul");
    }
}
})();
